package com.leyou.es;



import com.leyou.es.pojo.Item;
import com.leyou.es.repository.ItemRepository;
import org.elasticsearch.index.query.QueryBuilders;
import org.elasticsearch.search.aggregations.AggregationBuilders;
import org.elasticsearch.search.aggregations.Aggregations;
import org.elasticsearch.search.aggregations.bucket.terms.StringTerms;
import org.elasticsearch.search.sort.SortBuilders;
import org.elasticsearch.search.sort.SortOrder;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageRequest;
import org.springframework.data.elasticsearch.core.ElasticsearchTemplate;
import org.springframework.data.elasticsearch.core.aggregation.AggregatedPage;
import org.springframework.data.elasticsearch.core.query.FetchSourceFilter;
import org.springframework.data.elasticsearch.core.query.NativeSearchQueryBuilder;
import org.springframework.test.context.junit4.SpringRunner;

import java.util.ArrayList;
import java.util.List;

/**
 * @author :cza
 * @date :2020/5/12 12:29
 * @description :
 * @modyified By:
 */
@RunWith(SpringRunner.class)
@SpringBootTest
public class EsTest {

    @Autowired
    private ElasticsearchTemplate elasticsearchTemplate;
    @Autowired
    private ItemRepository itemRepository;
    //@Autowired
    //private ElasticsearchRestTemplate elasticsearchRestTemplate;
    @Test
    public  void testCreateIndex(){
        //创建索引库
        elasticsearchTemplate.createIndex(Item.class);
        //映射关系
        elasticsearchTemplate.putMapping(Item.class);
    /*  //创建索引库
        elasticsearchRestTemplate.createIndex(Item.class);
        //映射关系
        elasticsearchRestTemplate.putMapping(Item.class);*/
    }

    /**
     * 插上全部
     */
      @Test
    public  void insertIndexAll(){
          List<Item> list=new ArrayList<>();
          list.add(new Item(Integer.valueOf("1"),"坚果手机","手机","锤子",2699.00,"http://www.baidu.com"));
          list.add(new Item(Integer.valueOf("2"),"华为Mate10手机","手机","华为",3699.00,"http://www.baidu.com"));
          list.add(new Item(Integer.valueOf("3"),"小米Mi10手机","手机","小米",3099.00,"http://www.baidu.com"));
          list.add(new Item(Integer.valueOf("3"),"小米Mi8手机","手机","小米",2399.00,"http://www.baidu.com"));

          itemRepository.saveAll(list);
    }

    /**
     * 查询所有
     */
    @Test
    public  void testFind(){
        Iterable<Item> all = itemRepository.findAll();
        for (Item item :all){
            System.out.println("item= "+item);
        }
    }

    /**
     * 根据价格查询
     */
    @Test
    public  void testFindByPrice(){
        Iterable<Item> all = itemRepository.findByPriceBetween(2000d,3500d);
        for (Item item :all){
            System.out.println("item= "+item);
        }
    }
    @Test
    public void search(){
        //构建查询条件
        NativeSearchQueryBuilder queryBuilder=new NativeSearchQueryBuilder();
        //结果过滤
        queryBuilder.withSourceFilter(new FetchSourceFilter(new String[]{"id","title","price"},null));
        //添加基本分词查询
        queryBuilder.withQuery(QueryBuilders.matchQuery("title","小米"));
        //排序
        queryBuilder.withSort(SortBuilders.fieldSort("price").order(SortOrder.DESC));
        //分页
        queryBuilder.withPageable( PageRequest.of(1,20));
        Page<Item> items = this.itemRepository.search(queryBuilder.build());
        long totalcount = items.getTotalElements();
        System.out.println("totalcount="+totalcount);
        for (Item item:items){
            System.out.println(item);
        }
        List<Item> list = items.getContent();
        for (Item item :list){
            System.out.println("item="+list);
        }
    }

    @Test
    public  void testAgg(){
        NativeSearchQueryBuilder queryBuilder=new NativeSearchQueryBuilder();
        String aggName="popularBrand";
        queryBuilder.addAggregation(AggregationBuilders.terms(aggName).field("brand"));
        //查询并返回聚合结果
        AggregatedPage<Item> result = elasticsearchTemplate.queryForPage(queryBuilder.build(), Item.class);
        //解析聚合
        Aggregations aggs = result.getAggregations();

        //获取指定名称的聚合
        StringTerms terms = aggs.get(aggName);
        List<StringTerms.Bucket> buckets = terms.getBuckets();
        for (StringTerms.Bucket bucket:buckets){
            System.out.println("bucket.getKeyAsString="+bucket.getKeyAsString());
            System.out.println("bucket.getDocCount="+bucket.getDocCount());
        }
    }
}


